/*	Filename: apr_pool_base.h
	A base class for interfacing with APR pools
	
	Created	20050819	Brandon Fosdick <bfoz@bfoz.net>
		Moved from mod_dav_userdir.h

	Copyright Terran Development Co 2005 (BSD License)
*/

#include <unistd.h>

#include <iostream>

#include <apr_pools.h>

#ifndef	APR_POOL_BASE_H
#define	APR_POOL_BASE_H

//Overloaded operator new() for allocating from an APR pool
inline void* operator new(size_t t, apr_pool_t* p)
{
	return apr_pcalloc(p, t);
}

namespace apr
{
	#define	CAST_POOL_HANDLER(a)	reinterpret_cast<apr_status_t (*)(void*)>(a)
//	#define	CAST_POOL_HANDLER(a, T)	reinterpret_cast<apr_status_t (*)(void*)>(a<typename T>)

	//Templated destructor caller
	template<typename T>
	apr_status_t	pool_base_cleanup(T* ptr) { ptr->~T(); return OK; }

	
	//Base class for hooking into pool cleanup
	template<typename T>
	class pool_base
	{
//		#define	HANDLE_CLEANUP	CAST_POOL_HANDLER(pool_base_cleanup, T)
		apr_pool_t* pool;
		
		void register_cleanup()
		{
			if(pool == NULL)	//Nothing to do if no pool
				return;
			apr_status_t (*fptr)(T*) = &pool_base_cleanup<T>;	//Deal with gcc screwiness
			apr_pool_cleanup_register(pool, static_cast<void*>(this), CAST_POOL_HANDLER(fptr), apr_pool_cleanup_null);
		}
		
		void kill_cleanup()
		{
			if(pool == NULL)	//Nothing to do if no pool
				return;
			apr_status_t (*fptr)(T*) = &pool_base_cleanup<T>;	//Deal with gcc screwiness
			apr_pool_cleanup_kill(pool, static_cast<void*>(this), CAST_POOL_HANDLER(fptr));
		}
	public:
		pool_base(apr_pool_t* p=NULL) : pool(p)
		{
			register_cleanup();	//Register to be killed
		}
		~pool_base()	//Deregister on premature destruction
		{
			kill_cleanup();
		}
		apr_pool_t* get_pool() const { return pool; }
	protected:
		void set_pool(apr_pool_t* p)
		{
			if(pool == NULL)
				pool = p;	//If there is no pool of record, just store the new one
			else
			{
				kill_cleanup();		//Decline notification from the old pool
				pool = p;				//Store the new pool
				register_cleanup();	//Request notification from the new pool
			}
		}
	};
}

#endif
