/*	Filename: mod_log_dbd.cc
	Write access logs to a database using DBD
	
	Created	20061104	Brandon Fosdick <bfoz@bfoz.net>
	Copyright Brandon Fosdick 2006 (BSD License)
*/

//System includes
#include <string>

//Local includes
#include "hooks.h"
#include "mod_log_dbd.h"
#include "serverconfig.h"

//Macros for generating very simple configuration handlers
#define	HANDLE_CONFIG(a)	\
	const char* handle_config_##a(cmd_parms* parms, void* config, const char* arg1) \
	{ \
		ServerConfig *cfg = get_server_config(parms); \
		TRACE1(arg1); \
\
		if(cfg == NULL) \
			CERR("cfg = NULL"); \
\
		return cfg->set_##a(arg1); \
	}

#define	HANDLE_FLAG(a)	\
	const char* handle_config_##a(cmd_parms* parms, void* config, int arg1) \
	{ \
		ServerConfig *cfg = get_server_config(parms); \
		TRACE1(arg1); \
\
		if(cfg == NULL) \
			CERR("cfg = NULL"); \
\
		cfg->set_##a(arg1); \
		return NULL; \
	}

namespace log_dbd
{
	//Casting macros for registering hook/handler functions
	#define	CAST_HANDLER(a)	reinterpret_cast<const char*(*)()>(a)
	#define	HANDLER_FN(a)	CAST_HANDLER(handle_config_##a)

	HANDLE_CONFIG(dsn)			//Handle the DSN config directive
	HANDLE_CONFIG(format)		//Handle the format config directive

	//	List of configuration directives that can be in the httpd.conf file
	const command_rec commands[] = 
	{
		AP_INIT_TAKE1("LogDBD", HANDLER_FN(dsn), NULL, RSRC_CONF, "DSN in the form driver://user:password@hostname:port/schema or Off to disable"),
		AP_INIT_TAKE1("LogDBDFormat", HANDLER_FN(format), NULL, RSRC_CONF, "Fields to log"),
		{NULL}
	};

	//Create a server config record for the given server
	//	***	Even though server_rec is a list this function is called for each one
	//	***		That means the list is walked by core, and not by the module, 
	//	***		which is the exact opposite of hooks::child_init(). That's Apache for you.
	void* create_server_config(apr_pool_t* p, server_rec* s)
	{
		TRACE1(PTR_CHAR(PTR_METHOD(s, server_hostname)));
		
		return new(p) ServerConfig(s, p);
	}

	//This is called after a server config has been created for a virtual host
	//	If the virtual host config is missing anything now is the time to set it
	//		base_conf is the main server config
	//		new_conf is the config for the new virtual host
	//		return a pointer to the updated config (should be == to new_conf)
	void*	merge_server_config(apr_pool_t* p, void *base_conf, void* new_conf)
	{
		TRACE1(PTR_CHAR(get_server_config(base_conf)->get_server_hostname()) << " into " << PTR_CHAR(get_server_config(new_conf)->get_server_hostname()));
		CDBG_PTR(base_conf);
		CDBG_PTR(new_conf);
//		CDBG_PTR(get_server_config(base_conf)->get_server_hostname());
//		CDBG_PTR(get_server_config(new_conf)->get_server_hostname());
		
		if(new_conf != NULL)
			get_server_config(new_conf)->merge_from(get_server_config(base_conf));
		return new_conf;
	}

	extern "C"
	{
		module	AP_MODULE_DECLARE_DATA	log_dbd_module =
		{
			STANDARD20_MODULE_STUFF,
			NULL,
			NULL,
			create_server_config,
			merge_server_config,	//Required for VirtualHost support
			commands,				//	Configuration commands
			register_hooks,		//	Callback for registering hooks
		};
	};

};
