/*	Filename: mod_log_dbd.h
	Write access logs to a database using DBD
	
	Created	20061104	Brandon Fosdick <bfoz@bfoz.net>
	Copyright Brandon Fosdick 2006-2007 (BSD License)
*/

#ifndef	MOD_LOG_DBD_H
#define	MOD_LOG_DBD_H

//Apache includes
//	***	These must be local includes to resolve alloc conflict
#include "httpd.h"
#undef AP_HAVE_DESIGNATED_INITIALIZER
#include "http_config.h"

//System includes
#include <iostream>
#include <sstream>

#include "trace.h"

//Some debug output macros
//	** Apache redirects cerr to the error log
#define	LOG_PRE	std::cerr << PACKAGE << "/" << __FILE__ << "," << __LINE__ << " "
#define	LOG_POST	std::endl << std::flush
#define	LOG_WRAP(a)	{ LOG_PRE << a << LOG_POST; }

#define	ERR_PRE	"ERROR: " << PACKAGE << "/" << __FILE__ << "," << __LINE__ << " "
#define	ERR_WRAP(a)	{ std::ostringstream os; os << ERR_PRE << a << std::endl; std::cerr << os.str() << std::flush; }

#ifdef DEBUG
#	define	DEBUG_WRAP(a)	LOG_WRAP(a)
#else
#	define	DEBUG_WRAP(a)
#endif

//These are only for use with ostreams
#define	NULL_2_STR(a)	((a==NULL)?"NULL":"NOT NULL")
#define	PTR_METHOD(a,b)	((a==NULL)?"NULL":((a)->b))
#define	PTR_CHAR(a)	((a==NULL)?"NULL":(a))

#define	CDBG(a)	DEBUG_WRAP(": " << a)
#define	CDBG_VAR(a)	CDBG(#a " = " << (a))
#define	CDBG_PTR(a)	CDBG(#a " = " << NULL_2_STR(a))
#define	CDBG_ARR(a, b)	CDBG(#a "[" << b << "] = " << (a[b]))

#define	CERR(a)	ERR_WRAP(": " << a)
#define	CERR_VAR(a)	CERR(#a " = " << (a))
#define	CERR_PTR(a)	CERR(#a " = " << NULL_2_STR(a))

#define	CLOG(a)	LOG_WRAP(": " << a)
#define	CLOG_VAR(a)	CLOG(#a " = " << (a))
#define	CLOG_PTR(a)	CLOG(#a " = " << NULL_2_STR(a))
#define	CLOG_PTR_VAR(a)	CLOG(#a " = " << PTR_CHAR(a))

namespace log_dbd
{
	// first is bytes in, second is bytes out
	typedef	std::pair<apr_off_t, apr_off_t>	byte_count_t;

	//Forward declaration of module structure
	extern "C" module AP_MODULE_DECLARE_DATA log_dbd_module;

	//Convenience function
	inline byte_count_t* get_module_config(request_rec* r)
	{
		return static_cast<byte_count_t*>(ap_get_module_config(r->connection->conn_config, &log_dbd_module));
	}
}

#endif
